﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO;

// This is a demonstration of some simple graphics in WPF, including exporting a Canvas to a graphics file.

// The main window has a Canvas on it.  If you want the Canvas to be fixed-size, then set its MinHeight same
// as MaxHeight and MinWidth same as MaxWidth.  If the margins are also fixed, this makes the whole
// window fixed-size.

// If the Canvas does not have the ClipToBounds property set to true, then WPF will, as far as possible, draw
// shapes that extend beyond the Canvas into the surrounding Grid (which is the window itself).

// Many online examples went into this; if you recognize your own code, consider yourself thankfully
// acknowledged!

namespace GraphicsInWPF
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        public Window1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {
            // Draw a rectangle...
            double[] xvalues = new double[] { 50.0, 300.0, 300.0, 50.0, 50.0 };
            double[] yvalues = new double[] { 50.0, 50.0, 150.0, 150.0, 50.0 };
            DrawPolyline(canvas1, xvalues, yvalues, Brushes.Red, 1.0);
            
            // And a couple of lines, one of which runs off the screen...
            DrawLine(canvas1, 20, 20, 75, 100, Brushes.Blue, 5);
            DrawLine(canvas1, 40, 10, 90, 575, Brushes.Green, 3);

            // And a couple of dots
            DrawDot(canvas1, 20, 20, 10, Brushes.Plum);
            DrawDot(canvas1, 25, 10, 2, Brushes.Magenta); // very small dot

            // And some writing...
            DrawText(canvas1, 150, 80, DateTime.Now.ToLongTimeString());

            // And a line to point out the origin of the text block
            DrawLine(canvas1, 0, 0, 150, 80, Brushes.CornflowerBlue, 1);

            // Write it onto file(s)
            SaveAsJpeg(canvas1, 600, "c:\\Users\\Michael\\Desktop\\test.jpg");

            // And onto clipboard as a 96-dpi bitmap for pasting into downlevel applications
            CopyCanvasToClipboard(canvas1, 96);
        }

        // Procedures to draw things

        static void DrawDot(Panel p, double x, double y, double diameter, Brush brush)
        {
            var r = new Ellipse();
            r.Fill = brush;
            r.Height = diameter;
            r.Width = diameter;
            Canvas.SetLeft(r, x - 0.5*diameter);  // Ellipse has no X and Y properties of its own
            Canvas.SetTop(r, y - 0.5*diameter);
            p.Children.Add(r);
        }
        
        static void DrawLine(Panel p, double x1, double y1, double x2, double y2, Brush brush, double width)
        {
            var r = new Line();
            r.Stroke = brush;
            r.StrokeThickness = width;
            r.X1 = x1;
            r.Y1 = y1;
            r.X2 = x2;
            r.Y2 = y2;
            p.Children.Add(r);
        }

        static void DrawPolyline(Panel p, double[] x, double[] y, Brush brush, double width)
        {
            var r = new Polyline();
            r.Stroke = brush;
            r.StrokeThickness = width;
            if (x.Length != y.Length) throw new ArgumentException("X and Y arrays are not the same length");
            for (int i = 0; i < x.Count(); i++)
            {
                r.Points.Add(new Point(x[i], y[i]));
            }
            p.Children.Add(r);
        }

        static void DrawText(Panel p, double x, double y, string s)
        {
            TextBlock r = new TextBlock();
            r.Text = s;
            r.Foreground = Brushes.Brown;
            Canvas.SetLeft(r, x);
            Canvas.SetTop(r, y);
            r.FontFamily = new FontFamily("Tahoma");
            r.FontSize = 15;
            p.Children.Add(r);
        }
        
        static void SaveAsJpeg(Panel c, double dpi, string filename)
        {
            using (FileStream fs = new FileStream(filename, System.IO.FileMode.Create))
            {
                var enc = new JpegBitmapEncoder();
                enc.QualityLevel = 90;  // JPEG quality
                enc.Frames.Add(BitmapFrame.Create(MakeBitmap(c, dpi)));
                enc.Save(fs);
            }
        }

        //// Requires a ref to System.Drawing
        //static void CopyCanvasToClipboard(Panel c, double dpi)
        //{
        //    using (MemoryStream outStream = new MemoryStream())
        //    {
        //        var enc = new BmpBitmapEncoder();
        //        enc.Frames.Add(BitmapFrame.Create(MakeBitmap(c,dpi)));
        //        enc.Save(outStream);
        //        System.Drawing.Bitmap bitmap = new System.Drawing.Bitmap(outStream);
        //        Clipboard.SetDataObject(bitmap);
        //    }
        //}

        static void CopyCanvasToClipboard(Panel c, double dpi)
        {
            var d = new System.Windows.DataObject();
            d.SetData(
                System.Windows.DataFormats.Bitmap, 
                MakeBitmap(c,dpi), 
                true
             );
            System.Windows.Clipboard.SetDataObject(d, true);
        }

        static RenderTargetBitmap MakeBitmap(Panel c, double dpi)
        {
            // Deprive the canvas temporarily of its positioning and margins
            Transform tsave = c.LayoutTransform;
            c.LayoutTransform = null;
            Thickness msave = c.Margin;
            c.Margin = new Thickness(0, 0, msave.Right - msave.Left, msave.Bottom - msave.Top);

            // Prepare to render the canvas at 100% size in a rectangle the same size as itself
            Size size = new Size(c.ActualWidth, c.ActualHeight);
            c.Measure(size);
            c.Arrange(new Rect(size));

            // Render to a bitmap.  Note that pixel count is adjusted for dpi.
            var b =
                new RenderTargetBitmap(
                    (int)(size.Width * dpi / 96),
                    (int)(size.Height * dpi / 96),
                    dpi,
                    dpi,
                    PixelFormats.Default
                );
            b.Render(c);

            // Restore the positioning and margins of the canvas
            c.LayoutTransform = tsave;
            c.Margin = msave;

            return b;
        }



    }
}
